SystemOrganization addCategory: #'Magritte-Pharo3-Model'!

!GRPackage class methodsFor: '*Magritte-Pharo3-Model' stamp: 'lr 2/16/2010 09:16'!
magrittePharoModel
	^ self new
		name: 'Magritte-Pharo-Model';
		addDependency: 'Magritte-Model';
		url: #magritteUrl;
		yourself! !

!MorphicUIManager methodsFor: '*Magritte-Pharo3-Model' stamp: 'SeanDeNigris 9/29/2013 12:00'!
chooseFile: windowTitle in: directoryString

	| chooser |
	chooser := FileDialogWindow newWithTheme: UITheme current.
	chooser
		title: windowTitle;
		selectDirectory: directoryString;
		answerFileEntry.
	^ chooser openModal answer ifNotNil: [ :answer | answer asFileReference ].! !

Object subclass: #MAFileDatabase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Pharo3-Model'!

MAFileDatabase class
	instanceVariableNames: 'baseDirectory baseUrl'!

!MAFileDatabase commentStamp: '<historical>' prior: 0!
I provide locations for external files.

- The ==#baseDirectory== is the place where Magritte puts its file-database. Keep this value to nil to make it default to a subdirectory next to the Smalltalk image.
- The ==#baseUrl== is a nice optimization to allow Apache (or any other Web Server) to directly serve the files. ==#baseUrl== is an absolute URL-prefix that is used to generate the path to the file. If you have specified one the file data does not go trough the image anymore, but instead is directly served trough the properly configured Web Server.

The files are currently stored using the following scheme:

=/files/9d/bsy8kyp45g0q7blphknk48zujap2wd/earthmap1k.jpg
=1     2   3                            						  4

1. is the #baseDirectory as specified in the settings.
2. Are 256 directories named '00' to 'ff' to avoid having thousands of files in the same directory. Unfortunately this leads to problems with the Squeak file primitives and some filesystems don't handle that well. This part is generated at random.
3. This is a secure ID, similar to the Seaside session key. It is generated at random and provides a security system that even works through Apache (you have to disable directory listings of course): if you don't know the filename you can not access the file.
4. This is the original filename. Subclasses might want to store other cached versions of the same file there, for example resized images, etc.!

MAFileDatabase class
	instanceVariableNames: 'baseDirectory baseUrl'!

!MAFileDatabase class methodsFor: 'accessing' stamp: 'SeanDeNigris 9/29/2013 08:28'!
baseDirectory
	^ baseDirectory ifNil: [ FileLocator imageDirectory / 'files' ].! !

!MAFileDatabase class methodsFor: 'accessing' stamp: 'SeanDeNigris 9/29/2013 08:31'!
baseDirectory: aStringOrDirectory
	"Defines where the files are stored. If this value is set to nil, it defaults to a subdirectory of of the current image-location."

	baseDirectory := aStringOrDirectory asFileReference.! !

!MAFileDatabase class methodsFor: 'accessing' stamp: 'SeanDeNigris 9/29/2013 09:51'!
baseUrl
	^ baseUrl! !

!MAFileDatabase class methodsFor: 'accessing' stamp: 'DamienCassou 1/16/2014 09:55'!
baseUrl: aString
	"Defines the base-URL where the files are served from, when using an external web server. This setting is left to nil by default, causing the files to be served trough the image."

	baseUrl := aString isNil ifFalse: [
		aString last = $/
			ifFalse: [ aString ]
			ifTrue: [ aString copyUpToLast: $/ ] ]! !

!MAFileDatabase class methodsFor: 'public' stamp: 'SeanDeNigris 9/29/2013 08:53'!
garbageCollect
	"Remove obsolete files from the file-system that are not referenced from the image anymore. This method has to be called manually and is not intended to be portable."
	
	| mark sweep |
	mark := self allInstances
		collect: [ :each | each directory ].
	sweep := Array with: self baseDirectory.
	self locationDefinition do: [ :definition |
		sweep := sweep gather: [ :directory |
			directory hasChildren ifFalse: [ directory delete ].
			FileLocator imageDirectory directories select: [ :e | e basename size = definition first ] ] ].
	sweep do: [ :directory |
		(mark includes: directory) ifFalse: [ directory recursiveDelete ] ]! !

!MAFileDatabase class methodsFor: 'public' stamp: 'SeanDeNigris 9/29/2013 09:01'!
locationDefinition
	^ #( ( 2 '63450af8d9c2e17b' ) ( 30 'iaojv41bw67e0tud5m9rgplqfy8x3cs2kznh' ) )! !

!MAFileDatabase methodsFor: 'accessing' stamp: 'SeanDeNigris 9/29/2013 09:45'!
baseDirectory

	^ self class baseDirectory.! !

!MAFileDatabase methodsFor: 'accessing' stamp: 'DamienCassou 1/16/2014 09:57'!
baseUrl

	^ self class baseUrl.! !

!MAFileDatabase methodsFor: 'private' stamp: 'SeanDeNigris 9/29/2013 09:44'!
randomIdentifierName

	| digits idLength allowedCharacters |
	digits := $0 to: $9.
	allowedCharacters := digits, Character alphabet.
	idLength := 30.
	^ String streamContents: [ :str |
		idLength timesRepeat: [ str nextPut: allowedCharacters atRandom ] ].! !

!MAFileDatabase methodsFor: 'private' stamp: 'SeanDeNigris 9/29/2013 09:38'!
randomPartitionFolderName
	"Returns a string between '00' to 'FF'"

	| twoCharacterHexString |
	twoCharacterHexString := 16rFF atRandom printStringBase: 16 length: 2 padded: true.
	^ twoCharacterHexString asLowercase.! !

!MAFileDatabase methodsFor: 'public' stamp: 'StephanEggermont 3/4/2014 12:58'!
uniqueLocation
	"Finds an unique path to be used and create the necessary sub directories."

	| partition id result |
	[
		partition := self randomPartitionFolderName.
		id := self randomIdentifierName.
		result := self baseDirectory ensureCreateDirectory / partition / id.
	] doWhileTrue: [ result exists ].
	
	^ result ensureCreateDirectory.! !

Object subclass: #MAFileSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Pharo3-Model'!

!MAFileSystem class methodsFor: 'paths' stamp: 'SeanDeNigris 9/26/2013 13:01'!
extensionFor: aString 

	^ aString asFileReference extension.! !

!MAFileSystem class methodsFor: 'entries' stamp: 'SeanDeNigris 9/26/2013 12:49'!
fileSizeFor: filenameString 
	^ filenameString asFileReference size! !

!MAFileSystem class methodsFor: 'paths' stamp: 'SeanDeNigris 9/26/2013 12:37'!
fullNameFor: aString 

	^ aString asFileReference fullName.! !

!MAFileSystem class methodsFor: 'locations' stamp: 'SeanDeNigris 9/29/2013 12:14'!
imageDirectory

	^ FileLocator imageDirectory.! !

!MAFileSystem class methodsFor: 'mime types' stamp: 'SeanDeNigris 9/26/2013 13:13'!
mimeTypesForExtension: aString 

	| types |
	types := MIMEType forExtensionReturnMimeTypesOrNil: aString.
	^ types ifNotNil: [ types collect: [ :e | e asString ] ].! !

MAFileModel subclass: #MAExternalFileModel
	instanceVariableNames: 'location directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Pharo3-Model'!

!MAExternalFileModel commentStamp: '<historical>' prior: 0!
I manage the file-data I represent on the file-system. From the programmer this looks the same as if the file would be in memory (==*MAMemoryFileModel*==), as it is transparently loaded and written out as necessary.

I delegate my actual location on disk to MAFileDatabase (see class comment).!

!MAExternalFileModel class methodsFor: 'accessing' stamp: 'DiegoLont 2/4/2014 14:11'!
baseDirectoryPath
	" deprecated: use MAFileDatabase baseDirectoryPath instead "
	^MAFileDatabase baseDirectory fullName! !

!MAExternalFileModel class methodsFor: 'accessing' stamp: 'DiegoLont 7/31/2014 10:13'!
baseDirectoryPath: aString 
	" deprecated, but still used by pier "
	MAFileDatabase baseDirectory: aString! !

!MAExternalFileModel class methodsFor: 'accessing' stamp: 'DiegoLont 2/4/2014 14:23'!
baseUrl
	" deprecated: use MAFileDatabase baseDirectoryPath instead "
	^ MAFileDatabase baseUrl! !

!MAExternalFileModel class methodsFor: 'accessing' stamp: 'DiegoLont 7/31/2014 10:14'!
baseUrl: aString
	" deprecated: use MAFileDatabase baseUrl instead "
	^ MAFileDatabase baseUrl: aString! !

!MAExternalFileModel methodsFor: 'comparing' stamp: 'SeanDeNigris 9/29/2013 09:50'!
= anObject
    ^ super = anObject and: [ self directory = anObject directory ]! !

!MAExternalFileModel methodsFor: 'configuration' stamp: 'SeanDeNigris 9/29/2013 11:16'!
baseUrl
	^ self database baseUrl! !

!MAExternalFileModel methodsFor: 'accessing' stamp: 'DiegoLont 7/24/2013 14:01'!
contents
	| stream |
	^ self file exists
		ifFalse: [ ByteArray new ]
		ifTrue: [
			stream := self readStream.
			[ stream contents ]
				ensure: [ stream close ] ]! !

!MAExternalFileModel methodsFor: 'accessing' stamp: 'nfr 10/17/2007 12:26'!
contents: aByteArray
	| stream |
	stream := self writeStream.
	[ stream nextPutAll: aByteArray asByteArray ]
		ensure: [ stream close ].
	super contents: aByteArray! !

!MAExternalFileModel methodsFor: 'private' stamp: 'SeanDeNigris 9/29/2013 09:30'!
database

	^ MAFileDatabase new.! !

!MAExternalFileModel methodsFor: 'accessing-dynamic' stamp: 'SeanDeNigris 9/29/2013 09:49'!
directory

	^ directory ifNil: [ directory := self database uniqueLocation ].! !

!MAExternalFileModel methodsFor: 'accessing-dynamic' stamp: 'DiegoLont 7/24/2013 14:17'!
file
	^(self directory / self filename) asFileReference! !

!MAExternalFileModel methodsFor: 'initialization' stamp: 'DamienCassou 6/1/2014 12:12'!
finalize
	|  dir |
	dir := self directory.
	dir exists
		ifTrue: [ dir deleteAll ].
	[ (dir := dir parent) hasChildren ]
		whileFalse: [ dir ensureDelete ].
	super finalize.
	location := nil! !

!MAExternalFileModel methodsFor: 'comparing' stamp: 'SeanDeNigris 9/29/2013 09:53'!
hash
	^ super hash bitXor: self directory hash! !

!MAExternalFileModel methodsFor: 'copying' stamp: 'lr 3/28/2007 10:27'!
postCopy
	| previous |
	super postCopy.
	previous := self contents.
	location := nil.
	self contents: previous! !

!MAExternalFileModel methodsFor: 'accessing-dynamic' stamp: 'SeanDeNigris 9/29/2013 09:13'!
readStream
	^ self file readStream binary.! !

!MAExternalFileModel methodsFor: 'accessing-dynamic' stamp: 'SeanDeNigris 9/29/2013 09:15'!
writeStream
	^ self file writeStream binary! !

!GRPharoPlatform methodsFor: '*Magritte-Pharo3-Model' stamp: 'jf 9/16/2009 13:39'!
magritteAllSubInstancesOf: aClass do: aBlock
	"Evaluate the aBlock for all instances of aClass and all its subclasses."

	aClass allSubInstancesDo: aBlock! !

!GRPharoPlatform methodsFor: '*Magritte-Pharo3-Model' stamp: 'jf 9/16/2009 13:32'!
magritteClassNamed: aString
	"Return the class named aString, nil if the class can't be found."

	^ Smalltalk classNamed: aString! !

!GRPharoPlatform methodsFor: '*Magritte-Pharo3-Model' stamp: 'jf 9/15/2009 18:43'!
magritteColorClass
	"Return a Color class"
	
	^ Color! !

!GRPharoPlatform methodsFor: '*Magritte-Pharo3-Model' stamp: 'DamienCassou 11/21/2013 14:58'!
magritteEvaluatorClassFor: aClass
	"Answer an evaluator class appropriate for evaluating expressions in the 
	context of this class."
	
	^ aClass compilerClass! !

!GRPharoPlatform methodsFor: '*Magritte-Pharo3-Model' stamp: 'JohnCBorden 4/26/2015 13:42'!
magritteTimeStampClass
	"Return the platform's TimeStamp class. It is currently assumed that all platforms
	have one, though this may not be correct and could require moving timestamp stuff
	to its own package in order to resolve."
	
	^ DateAndTime! !

!GRPharoPlatform methodsFor: '*Magritte-Pharo3-Model' stamp: 'JohnCBorden 4/26/2015 13:42'!
magritteTimeStampIfAbsent: absentBlock
	"Return the TimeStamp class, or if the platform does not have a TimeStamp
	return the result of evaluating absentBlock."
	
	^ DateAndTime! !

!GRPharoPlatform methodsFor: '*Magritte-Pharo3-Model' stamp: 'lr 1/11/2010 23:04'!
magritteUniqueObject
	"Answer a random object, such as a UUID, that is extremely likely to
	be unique over space and time."

	^ ByteArray withAll: UUID new! !
